/* Copyright (C) 1991, 1993 Free Software Foundation, Inc.
This file is part of the GNU C Library.

The GNU C Library is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

The GNU C Library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GNU C Library; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include <errno.h>

static char iferrno[] = "#ifdef _ERRNO_H";
static char endiferrno[] = "#endif /* <errno.h> included.  */";
static char ifEmath[] = "#if !defined(__Emath_defined) && \
 (defined(_ERRNO_H) || defined(__need_Emath))";
static char endifEmath[] = "#endif /* Emath not defined and <errno.h> \
included or need Emath.  */";

static int biggest_value = 0;
static int done_ENOSYS = 0;
static int done_ERANGE = 0, done_EDOM = 0;

static void
DO(name, value)
     char *name;
     int value;
{
  int is_ERANGE = !done_ERANGE && !strcmp(name, "ERANGE");
  int is_EDOM = !done_EDOM && !strcmp(name, "EDOM");
  int is_Emath = is_ERANGE || is_EDOM;

  if (is_Emath)
    {
      puts(endiferrno);
      puts(ifEmath);
    }

  if (!strcmp (name, "EWOULDBLOCK"))
    {
      puts ("#define EWOULDBLOCK EAGAIN /* Translated in glibc. */");
      name = "EWOULDBLOCK_sys /* Value actually returned by kernel. */";
    }

  printf ("#define %s %d\n", name, value);

  if (is_Emath)
    {
      puts(endifEmath);
      puts(iferrno);
    }

  if (value > biggest_value)
    biggest_value = value;

  if (is_ERANGE)
    done_ERANGE = 1;
  else if (is_EDOM)
    done_EDOM = 1;
  else if (!done_ENOSYS && !strcmp(name, "ENOSYS"))
    done_ENOSYS = 1;
}

int
main()
{
  puts(iferrno);

#ifdef EPERM
  DO("EPERM", EPERM);
#endif
#ifdef ENOENT
  DO("ENOENT", ENOENT);
#endif
#ifdef ESRCH
  DO("ESRCH", ESRCH);
#endif
#ifdef EINTR
  DO("EINTR", EINTR);
#endif
#ifdef EIO
  DO("EIO", EIO);
#endif
#ifdef ENXIO
  DO("ENXIO", ENXIO);
#endif
#ifdef E2BIG
  DO("E2BIG", E2BIG);
#endif
#ifdef ENOEXEC
  DO("ENOEXEC", ENOEXEC);
#endif
#ifdef EBADF
  DO("EBADF", EBADF);
#endif
#ifdef ECHILD
  DO("ECHILD", ECHILD);
#endif
#ifdef EAGAIN
  DO("EAGAIN", EAGAIN);
#endif
#ifdef ENOMEM
  DO("ENOMEM", ENOMEM);
#endif
#ifdef EACCES
  DO("EACCES", EACCES);
#endif
#ifdef EFAULT
  DO("EFAULT", EFAULT);
#endif
#ifdef ENOTBLK
  DO("ENOTBLK", ENOTBLK);
#endif
#ifdef EBUSY
  DO("EBUSY", EBUSY);
#endif
#ifdef EEXIST
  DO("EEXIST", EEXIST);
#endif
#ifdef EXDEV
  DO("EXDEV", EXDEV);
#endif
#ifdef ENODEV
  DO("ENODEV", ENODEV);
#endif
#ifdef ENOTDIR
  DO("ENOTDIR", ENOTDIR);
#endif
#ifdef EISDIR
  DO("EISDIR", EISDIR);
#endif
#ifdef EINVAL
  DO("EINVAL", EINVAL);
#endif
#ifdef ENFILE
  DO("ENFILE", ENFILE);
#endif
#ifdef EMFILE
  DO("EMFILE", EMFILE);
#endif
#ifdef ENOTTY
  DO("ENOTTY", ENOTTY);
#endif
#ifdef ETXTBSY
  DO("ETXTBSY", ETXTBSY);
#endif
#ifdef EFBIG
  DO("EFBIG", EFBIG);
#endif
#ifdef ENOSPC
  DO("ENOSPC", ENOSPC);
#endif
#ifdef ESPIPE
  DO("ESPIPE", ESPIPE);
#endif
#ifdef EROFS
  DO("EROFS", EROFS);
#endif
#ifdef EMLINK
  DO("EMLINK", EMLINK);
#endif
#ifdef EPIPE
  DO("EPIPE", EPIPE);
#endif
#ifdef EDOM
  DO("EDOM", EDOM);
#endif
#ifdef ERANGE
  DO("ERANGE", ERANGE);
#endif
#ifdef EWOULDBLOCK
  DO("EWOULDBLOCK", EWOULDBLOCK);
#endif
#ifdef EDEADLK
  DO("EDEADLK", EDEADLK);
#endif
#ifdef EINPROGRESS
  DO("EINPROGRESS", EINPROGRESS);
#endif
#ifdef EALREADY
  DO("EALREADY", EALREADY);
#endif
#ifdef ENOTSOCK
  DO("ENOTSOCK", ENOTSOCK);
#endif
#ifdef EDESTADDRREQ
  DO("EDESTADDRREQ", EDESTADDRREQ);
#endif
#ifdef EMSGSIZE
  DO("EMSGSIZE", EMSGSIZE);
#endif
#ifdef EPROTOTYPE
  DO("EPROTOTYPE", EPROTOTYPE);
#endif
#ifdef ENOPROTOOPT
  DO("ENOPROTOOPT", ENOPROTOOPT);
#endif
#ifdef EPROTONOSUPPORT
  DO("EPROTONOSUPPORT", EPROTONOSUPPORT);
#endif
#ifdef ESOCKTNOSUPPORT
  DO("ESOCKTNOSUPPORT", ESOCKTNOSUPPORT);
#endif
#ifdef EOPNOTSUPP
  DO("EOPNOTSUPP", EOPNOTSUPP);
#endif
#ifdef EPFNOSUPPORT
  DO("EPFNOSUPPORT", EPFNOSUPPORT);
#endif
#ifdef EAFNOSUPPORT
  DO("EAFNOSUPPORT", EAFNOSUPPORT);
#endif
#ifdef EADDRINUSE
  DO("EADDRINUSE", EADDRINUSE);
#endif
#ifdef EADDRNOTAVAIL
  DO("EADDRNOTAVAIL", EADDRNOTAVAIL);
#endif
#ifdef ENETDOWN
  DO("ENETDOWN", ENETDOWN);
#endif
#ifdef ENETUNREACH
  DO("ENETUNREACH", ENETUNREACH);
#endif
#ifdef ENETRESET
  DO("ENETRESET", ENETRESET);
#endif
#ifdef ECONNABORTED
  DO("ECONNABORTED", ECONNABORTED);
#endif
#ifdef ECONNRESET
  DO("ECONNRESET", ECONNRESET);
#endif
#ifdef ENOBUFS
  DO("ENOBUFS", ENOBUFS);
#endif
#ifdef EISCONN
  DO("EISCONN", EISCONN);
#endif
#ifdef ENOTCONN
  DO("ENOTCONN", ENOTCONN);
#endif
#ifdef ESHUTDOWN
  DO("ESHUTDOWN", ESHUTDOWN);
#endif
#ifdef ETOOMANYREFS
  DO("ETOOMANYREFS", ETOOMANYREFS);
#endif
#ifdef ETIMEDOUT
  DO("ETIMEDOUT", ETIMEDOUT);
#endif
#ifdef ECONNREFUSED
  DO("ECONNREFUSED", ECONNREFUSED);
#endif
#ifdef ELOOP
  DO("ELOOP", ELOOP);
#endif
#ifdef ENAMETOOLONG
  DO("ENAMETOOLONG", ENAMETOOLONG);
#endif
#ifdef EHOSTDOWN
  DO("EHOSTDOWN", EHOSTDOWN);
#endif
#ifdef EHOSTUNREACH
  DO("EHOSTUNREACH", EHOSTUNREACH);
#endif
#ifdef ENOTEMPTY
  DO("ENOTEMPTY", ENOTEMPTY);
#endif
#ifdef EPROCLIM
  DO("EPROCLIM", EPROCLIM);
#endif
#ifdef EUSERS
  DO("EUSERS", EUSERS);
#endif
#ifdef EDQUOT
  DO("EDQUOT", EDQUOT);
#endif
#ifdef EPERM
  DO("EPERM", EPERM);
#endif
#ifdef ENOENT
  DO("ENOENT", ENOENT);
#endif
#ifdef ESRCH
  DO("ESRCH", ESRCH);
#endif
#ifdef EINTR
  DO("EINTR", EINTR);
#endif
#ifdef EIO
  DO("EIO", EIO);
#endif
#ifdef ENXIO
  DO("ENXIO", ENXIO);
#endif
#ifdef E2BIG
  DO("E2BIG", E2BIG);
#endif
#ifdef ENOEXEC
  DO("ENOEXEC", ENOEXEC);
#endif
#ifdef EBADF
  DO("EBADF", EBADF);
#endif
#ifdef ECHILD
  DO("ECHILD", ECHILD);
#endif
#ifdef EAGAIN
  DO("EAGAIN", EAGAIN);
#endif
#ifdef ENOMEM
  DO("ENOMEM", ENOMEM);
#endif
#ifdef EACCES
  DO("EACCES", EACCES);
#endif
#ifdef EFAULT
  DO("EFAULT", EFAULT);
#endif
#ifdef ENOTBLK
  DO("ENOTBLK", ENOTBLK);
#endif
#ifdef EBUSY
  DO("EBUSY", EBUSY);
#endif
#ifdef EEXIST
  DO("EEXIST", EEXIST);
#endif
#ifdef EXDEV
  DO("EXDEV", EXDEV);
#endif
#ifdef ENODEV
  DO("ENODEV", ENODEV);
#endif
#ifdef ENOTDIR
  DO("ENOTDIR", ENOTDIR);
#endif
#ifdef EISDIR
  DO("EISDIR", EISDIR);
#endif
#ifdef EINVAL
  DO("EINVAL", EINVAL);
#endif
#ifdef ENFILE
  DO("ENFILE", ENFILE);
#endif
#ifdef EMFILE
  DO("EMFILE", EMFILE);
#endif
#ifdef ENOTTY
  DO("ENOTTY", ENOTTY);
#endif
#ifdef ETXTBSY
  DO("ETXTBSY", ETXTBSY);
#endif
#ifdef EFBIG
  DO("EFBIG", EFBIG);
#endif
#ifdef ENOSPC
  DO("ENOSPC", ENOSPC);
#endif
#ifdef ESPIPE
  DO("ESPIPE", ESPIPE);
#endif
#ifdef EROFS
  DO("EROFS", EROFS);
#endif
#ifdef EMLINK
  DO("EMLINK", EMLINK);
#endif
#ifdef EPIPE
  DO("EPIPE", EPIPE);
#endif
#ifdef EDOM
  DO("EDOM", EDOM);
#endif
#ifdef ERANGE
  DO("ERANGE", ERANGE);
#endif
#ifdef EWOULDBLOCK
  DO("EWOULDBLOCK", EWOULDBLOCK);
#endif
#ifdef EDEADLK
  DO("EDEADLK", EDEADLK);
#endif
#ifdef EINPROGRESS
  DO("EINPROGRESS", EINPROGRESS);
#endif
#ifdef EALREADY
  DO("EALREADY", EALREADY);
#endif
#ifdef ENOTSOCK
  DO("ENOTSOCK", ENOTSOCK);
#endif
#ifdef EDESTADDRREQ
  DO("EDESTADDRREQ", EDESTADDRREQ);
#endif
#ifdef EMSGSIZE
  DO("EMSGSIZE", EMSGSIZE);
#endif
#ifdef EPROTOTYPE
  DO("EPROTOTYPE", EPROTOTYPE);
#endif
#ifdef ENOPROTOOPT
  DO("ENOPROTOOPT", ENOPROTOOPT);
#endif
#ifdef EPROTONOSUPPORT
  DO("EPROTONOSUPPORT", EPROTONOSUPPORT);
#endif
#ifdef ESOCKTNOSUPPORT
  DO("ESOCKTNOSUPPORT", ESOCKTNOSUPPORT);
#endif
#ifdef EOPNOTSUPP
  DO("EOPNOTSUPP", EOPNOTSUPP);
#endif
#ifdef EPFNOSUPPORT
  DO("EPFNOSUPPORT", EPFNOSUPPORT);
#endif
#ifdef EAFNOSUPPORT
  DO("EAFNOSUPPORT", EAFNOSUPPORT);
#endif
#ifdef EADDRINUSE
  DO("EADDRINUSE", EADDRINUSE);
#endif
#ifdef EADDRNOTAVAIL
  DO("EADDRNOTAVAIL", EADDRNOTAVAIL);
#endif
#ifdef ENETDOWN
  DO("ENETDOWN", ENETDOWN);
#endif
#ifdef ENETUNREACH
  DO("ENETUNREACH", ENETUNREACH);
#endif
#ifdef ENETRESET
  DO("ENETRESET", ENETRESET);
#endif
#ifdef ECONNABORTED
  DO("ECONNABORTED", ECONNABORTED);
#endif
#ifdef ECONNRESET
  DO("ECONNRESET", ECONNRESET);
#endif
#ifdef ENOBUFS
  DO("ENOBUFS", ENOBUFS);
#endif
#ifdef EISCONN
  DO("EISCONN", EISCONN);
#endif
#ifdef ENOTCONN
  DO("ENOTCONN", ENOTCONN);
#endif
#ifdef ESHUTDOWN
  DO("ESHUTDOWN", ESHUTDOWN);
#endif
#ifdef ETOOMANYREFS
  DO("ETOOMANYREFS", ETOOMANYREFS);
#endif
#ifdef ETIMEDOUT
  DO("ETIMEDOUT", ETIMEDOUT);
#endif
#ifdef ECONNREFUSED
  DO("ECONNREFUSED", ECONNREFUSED);
#endif
#ifdef ELOOP
  DO("ELOOP", ELOOP);
#endif
#ifdef ENAMETOOLONG
  DO("ENAMETOOLONG", ENAMETOOLONG);
#endif
#ifdef EHOSTDOWN
  DO("EHOSTDOWN", EHOSTDOWN);
#endif
#ifdef EHOSTUNREACH
  DO("EHOSTUNREACH", EHOSTUNREACH);
#endif
#ifdef ENOTEMPTY
  DO("ENOTEMPTY", ENOTEMPTY);
#endif
#ifdef EPROCLIM
  DO("EPROCLIM", EPROCLIM);
#endif
#ifdef EUSERS
  DO("EUSERS", EUSERS);
#endif
#ifdef EDQUOT
  DO("EDQUOT", EDQUOT);
#endif

  if (!done_EDOM || !done_ERANGE)
    {
      puts(endiferrno);
      puts(ifEmath);
      if (!done_EDOM)
	printf("#define EDOM %d\n", ++biggest_value);
      if (!done_ERANGE)
	printf("#define ERANGE %d\n", ++biggest_value);
      puts(endifEmath);
    }

  if (!done_ENOSYS)
    printf("#define ENOSYS %d\n", ++biggest_value);

  puts(endiferrno);

  puts("#undef __need_Emath");
  puts("#ifndef __Emath_defined\n#define __Emath_defined 1\n#endif");

  exit(0);
}
